//****************************************************************************
// DMXPWM.c
// Receives DMX Serial Data and converts to RGB PWM Led Output
// PIC12F1572
// 16 Mhz Internal
// RA0, RA1, RA2 are PWM Outputs
// RA5 is Uart Receieve Pin
// Tested by RCI
// 7-9-2015
//****************************************************************************
#define RREG PWM2DC
#define GREG PWM1DC
#define BREG PWM3DC

unsigned int timeout, cnt, pbhold;
const Channels = 3;            // Number of Channels, used by the device
char Text[4];                  // Text array for ByteToStr conversation
char DMX_Byte;                 // DMX read value
int DMX_Channel;               // Number of DMX channel (up to 512)
int FirstDeviceCh;             // The first channel, used by device
int LastDeviceCh;              // The last channel, used by device
char DMX_Array[Channels];      // Channel width.
unsigned char DMXStartingAddress;
unsigned char DMXFlag;
//****************************************************************************
void interrupt()
{
  if (PIR1.RCIF == 1)        // If UART interrupt enabled
  {
    if(RCSTA.FERR)           // Framing error bit
      {
      DMX_Byte = RCREG;      // Update RCREG register and receiving next byte
      DMX_Channel = 0;       // Start transmition detection
      }
      else
         {
         if((DMX_Channel >= FirstDeviceCh) && (DMX_Channel < LastDeviceCh))
            {
            DMX_Array[DMX_Channel - FirstDeviceCh] = RCREG;
            }
            else
              {
              DMX_Byte = RCREG;
              }
         DMX_Channel++;
         }

    PIR1.RCIF = 0;            // Reset UART interrupt
  }
}
//**************************************************************************
void Init_PWM()
{
  CWG1CON0 = 0;
  // SLEW RATE
  SLRCONA.SLRA0 = 0;
  SLRCONA.SLRA1 = 0;
  SLRCONA.SLRA2 = 0;

  PWM1CON    = 0b11000000;      // Enable PWM Module,  Output Enable Bit ON
  PWM1CLKCON = 0b00110001;      // Divide by 8, HFINTOSC
  PWM2CON    = 0b11000000;      // Enable PWM Module,  Output Enable Bit ON
  PWM2CLKCON = 0b00110001;      // Divide by 8, HFINTOSC
  PWM3CON    = 0b11000000;      // Enable PWM Module,  Output Enable Bit ON
  PWM3CLKCON = 0b00110001;      // Divide by 8, HFINTOSC

  PWM1PH = 0;     // phase
  PWM2PH = 0;
  PWM3PH = 0;
  PWM1DC = 0;     // duty cycle
  PWM2DC = 0;
  PWM3DC = 0;
  PWM1PR = 30000; // period
  PWM2PR = 30000;
  PWM3PR = 30000;
}
//**************************************************************************
void pwmUpdate()
{
  PWMLD.PWM1LDA_A = 1;
  PWMLD.PWM2LDA_A = 1;
  PWMLD.PWM3LDA_A = 1;
}
//**************************************************************************
void PWMColor(unsigned char RedVal, unsigned char GreenVal, unsigned char BlueVal)
{
  RREG = RedVal * 255;
  GREG = GreenVal * 255;
  BREG = BlueVal * 255;
  pwmUpdate();
}

//****************************************************************************
void InitDMXPort()
{
  SPBRGH = 0;
  SPBRGL = 3;       // Set speed 250000 bps / 16 Mhz)
  TXSTA.BRGH = 1;   // Set High speed
  TXSTA.SYNC = 0;   // Enable Asynchronous mode
  PIE1 = 0;         // Peripheral interrupt enable
  PIE1.RCIE = 1;    // Enable interrupt on UART receive
  RCSTA.RX9 = 1;    // Enable 9th bit receive
  DMX_Byte = RCREG; // Read 1st byte from port
  DMX_Byte = RCREG; // Read 2nd byte from port
  DMX_Byte = RCREG; // Read 3rd byte from port
  RCSTA.CREN = 1;   // Enable continuous receive
  RCSTA.SPEN = 1;   // Serial port enable(configure RX and TX pins as port pins)
  RCSTA.ADDEN = 0;  // Disable channel detection
  INTCON.PEIE = 1;  // Enable peripheral interrupts
  INTCON.GIE = 1;   // Enable all interrupts
}
//****************************************************************************
void main() 
{
  ANSELA = 0b00000000;         // All Analog Inputs Disabled
  PORTA  = 0b00000000;         // All Port Pins Off
  TRISA  = 0b00110000;         // RA5 Setup as Input
  OSCCON = 0b01111010;         // Internal 16Mhz Oscillator
  DMX_Channel = 0;             // Set initial channel
  CM1CON0.C1ON = 0;            // Disable Comparator
  INTCON.GIE = 1;              // Enable Global Interrupts
  INTCON.PEIE = 1;             // Enable Peripheral Interrupts

  APFCON.RXDTSEL = 1;          // Sets up RX Input as RA5
  APFCON.P1SEL = 0;            // Sets up RA1 as PWM Input
  Init_PWM();                  // Initialize PWM Setup
  InitDMXPort();               // Initialize DMX Serial Setup
  // Set DMX Address
  DMXStartingAddress = 1;      // Set starting DMX channel number.
  FirstDeviceCh = DMXStartingAddress;
  LastDeviceCh = FirstDeviceCh + Channels; // Calculate last DMX channel number.

//****************************************************************************

  while (1 == 1)
  {
      PWMColor(DMX_Array[0], DMX_Array[1], DMX_Array[2]);
  }
}
//****************************************************************************